@@ -3,15 +3,22 @@ package Crypt::Perl::ECDSA::KeyBase;
 use strict;
 use warnings;
 
+use parent qw(
+    Crypt::Perl::KeyBase
+);
+
 use Crypt::Format ();
 use Module::Load ();
 
 use Crypt::Perl::ASN1 ();
+use Crypt::Perl::BigInt ();
 use Crypt::Perl::Math ();
 use Crypt::Perl::ECDSA::EC::Curve ();
+use Crypt::Perl::ECDSA::EC::DB ();
 use Crypt::Perl::ECDSA::EC::Point ();
 use Crypt::Perl::ECDSA::ECParameters ();
 use Crypt::Perl::ECDSA::Utils ();
+use Crypt::Perl::X ();
 
 use constant OID_ecPublicKey => '1.2.840.10045.2.1';
 
@@ -29,47 +36,66 @@ use constant ASN1_Params => Crypt::Perl::ECDSA::ECParameters::ASN1_ECParameters(
     }
 >;
 
+use constant _JWK_THUMBPRINT_JSON_ORDER => qw( crv kty x y );
+
+use constant JWA_DIGEST_prime256v1 => 'sha256';
+use constant JWA_DIGEST_secp384r1 => 'sha384';
+use constant JWA_DIGEST_secp521r1 => 'sha512';
+
+use constant JWA_CURVE_ALG_prime256v1 => 'ES256';
+use constant JWA_CURVE_ALG_secp384r1 => 'ES384';
+use constant JWA_CURVE_ALG_secp521r1 => 'ES512';
+
+use constant JWK_CURVE_prime256v1 => 'P-256';
+use constant JWK_CURVE_secp384r1 => 'P-384';
+use constant JWK_CURVE_secp521r1 => 'P-512';
+
+#Expects $key_parts to be a hash ref:
+#
+#   version - AFAICT unused
+#   private - BigInt or its byte-string representation
+#   public  - ^^
+#
+sub new_by_curve_name {
+    my ($class, $key_parts, $curve_name) = @_;
+
+    #We could store the curve name on here if looking it up
+    #in to_der_with_curve_name() proves prohibitive.
+    return $class->new(
+        $key_parts,
+
+        #“Fake out” the $curve_parts attribute by recreating
+        #the structure that ASN.1 would give from a named curve.
+        {
+            namedCurve => Crypt::Perl::ECDSA::EC::DB::get_oid_for_curve_name($curve_name),
+        },
+    );
+}
+
 #$msg has to be small enough that the key could have signed it.
 #It’s probably a digest rather than the original message.
 sub verify {
     my ($self, $msg, $sig) = @_;
 
     my $struct = Crypt::Perl::ASN1->new()->prepare(ASN1_SIGNATURE)->decode($sig);
-    my ($r, $s) = @{$struct}{ qw( r s ) };
 
-    if ($r >= 1 && $s >= 1) {
-        my ($x, $y) = Crypt::Perl::ECDSA::Utils::split_G_or_public( $self->{'public'}->as_bytes() );
-        $_ = Crypt::Perl::BigInt->from_bytes($_) for ($x, $y);
+    return $self->_verify($msg, @{$struct}{ qw( r s ) });
+}
 
-        my $curve = $self->_get_curve_obj();
+#cf. RFC 7518, page 8
+sub verify_jwa {
+    my ($self, $msg, $sig) = @_;
 
-        my $Q = Crypt::Perl::ECDSA::EC::Point->new(
-            $curve,
-            $curve->from_bigint($x),
-            $curve->from_bigint($y),
-        );
+    my $dgst_cr = $self->_get_jwk_digest_cr();
 
-        my $e = Crypt::Perl::BigInt->from_bytes($msg);
+    my $half_len = (length $sig) / 2;
 
-        #----------------------------------------------------------------------
+    my $r = substr($sig, 0, $half_len);
+    my $s = substr($sig, $half_len);
 
-        my $n = $self->_curve()->{'n'};
+    $_ = Crypt::Perl::BigInt->from_bytes($_) for ($r, $s);
 
-        if ($r < $n && $s < $n) {
-            my $c = $s->copy()->bmodinv($n);
-
-            my $u1 = ($e * $c) % $n;
-            my $u2 = ($r * $c) % $n;
-
-            my $point = $self->_G()->multiply($u1)->add( $Q->multiply($u2) );
-
-            my $v = $point->get_x()->to_bigint() % $n;
-
-            return 1 if $v == $r;
-        }
-    }
-
-    return 0;
+    return $self->_verify($dgst_cr->($msg), $r, $s);
 }
 
 sub to_der_with_curve_name {
@@ -112,6 +138,99 @@ sub get_curve_name {
     return Crypt::Perl::ECDSA::EC::DB::get_curve_name_by_data( $self->_curve() );
 }
 
+sub get_struct_for_public_jwk {
+    my ($self) = @_;
+
+    my ($xb, $yb) = Crypt::Perl::ECDSA::Utils::split_G_or_public( $self->{'public'}->as_bytes() );
+
+    Module::Load::load('MIME::Base64');
+
+    return {
+        kty => 'EC',
+        crv => $self->_get_jwk_curve_name(),
+        x => MIME::Base64::encode_base64url($xb),
+        y => MIME::Base64::encode_base64url($yb),
+    }
+}
+
+sub get_jwa_alg {
+    my ($self) = @_;
+
+    my $name = $self->get_curve_name();
+
+    my $getter_cr = __PACKAGE__->can("JWA_CURVE_ALG_$name") or do {
+        die sprintf( "“%s” knows of no JWA “alg” for the curve “%s”!", ref($self), $name);
+    };
+
+    return $getter_cr->();
+}
+
+#----------------------------------------------------------------------
+
+sub _get_jwk_digest_cr {
+    my ($self) = @_;
+
+    my $name = $self->get_curve_name();
+
+    my $getter_cr = $self->can("JWA_DIGEST_$name") or do {
+        die sprintf( "“%s” knows of no digest to use for JWA with the curve “%s”!", ref($self), $name);
+    };
+
+    Module::Load::load('Digest::SHA');
+
+    return Digest::SHA->can( $getter_cr->() );
+}
+
+sub _get_jwk_curve_name {
+    my ($self) = @_;
+
+    my $name = $self->get_curve_name();
+
+    my $getter_cr = __PACKAGE__->can("JWK_CURVE_$name") or do {
+        die sprintf( "“%s” knows of no JWK “crv” for the curve “%s”!", ref($self), $name);
+    };
+
+    return $getter_cr->();
+}
+
+sub _verify {
+    my ($self, $msg, $r, $s) = @_;
+
+    if ($r->bge(1) && $s->bge(1)) {
+        my ($x, $y) = Crypt::Perl::ECDSA::Utils::split_G_or_public( $self->{'public'}->as_bytes() );
+        $_ = Crypt::Perl::BigInt->from_bytes($_) for ($x, $y);
+
+        my $curve = $self->_get_curve_obj();
+
+        my $Q = Crypt::Perl::ECDSA::EC::Point->new(
+            $curve,
+            $curve->from_bigint($x),
+            $curve->from_bigint($y),
+        );
+
+        my $e = Crypt::Perl::BigInt->from_bytes($msg);
+
+        #----------------------------------------------------------------------
+
+        my $n = $self->_curve()->{'n'};
+
+        if ($r < $n && $s < $n) {
+            my $c = $s->copy()->bmodinv($n);
+
+            my $u1 = $e->copy()->bmul($c)->bmod($n);
+            my $u2 = $r->copy()->bmul($c)->bmod($n);
+
+            my $point = $self->_G()->multiply($u1)->add( $Q->multiply($u2) );
+
+            my $v = $point->get_x()->to_bigint()->copy()->bmod($n);
+
+            return 1 if $v->beq($r);
+        }
+    }
+
+    return 0;
+}
+
 #return isa EC::Point
 sub _G {
     my ($self) = @_;
@@ -218,7 +337,6 @@ sub _add_params {
 sub _curve_params_for_OID {
     my ($self, $oid) = @_;
 
-    require Crypt::Perl::ECDSA::EC::DB;
     return Crypt::Perl::ECDSA::EC::DB::get_curve_data_by_oid($oid);
 }
 
